@@ -1,6 +1,6 @@
 # bash completion for restic                               -*- shell-script -*-
 
-__debug()
+__restic_debug()
 {
     if [[ -n ${BASH_COMP_DEBUG_FILE} ]]; then
         echo "$*" >> "${BASH_COMP_DEBUG_FILE}"
@@ -9,13 +9,13 @@ __debug()
 
 # Homebrew on Macs have version 1.3 of bash-completion which doesn't include
 # _init_completion. This is a very minimal version of that function.
-__my_init_completion()
+__restic_init_completion()
 {
     COMPREPLY=()
     _get_comp_words_by_ref "$@" cur prev words cword
 }
 
-__index_of_word()
+__restic_index_of_word()
 {
     local w word=$1
     shift
@@ -27,7 +27,7 @@ __index_of_word()
     index=-1
 }
 
-__contains_word()
+__restic_contains_word()
 {
     local w word=$1; shift
     for w in "$@"; do
@@ -36,9 +36,9 @@ __contains_word()
     return 1
 }
 
-__handle_reply()
+__restic_handle_reply()
 {
-    __debug "${FUNCNAME[0]}"
+    __restic_debug "${FUNCNAME[0]}"
     case $cur in
         -*)
             if [[ $(type -t compopt) = "builtin" ]]; then
@@ -62,8 +62,8 @@ __handle_reply()
                 fi
 
                 local index flag
-                flag="${cur%%=*}"
-                __index_of_word "${flag}" "${flags_with_completion[@]}"
+                flag="${cur%=*}"
+                __restic_index_of_word "${flag}" "${flags_with_completion[@]}"
                 COMPREPLY=()
                 if [[ ${index} -ge 0 ]]; then
                     PREFIX=""
@@ -81,7 +81,7 @@ __handle_reply()
 
     # check if we are handling a flag with special work handling
     local index
-    __index_of_word "${prev}" "${flags_with_completion[@]}"
+    __restic_index_of_word "${prev}" "${flags_with_completion[@]}"
     if [[ ${index} -ge 0 ]]; then
         ${flags_completion[${index}]}
         return
@@ -114,24 +114,30 @@ __handle_reply()
     if declare -F __ltrim_colon_completions >/dev/null; then
         __ltrim_colon_completions "$cur"
     fi
+
+    # If there is only 1 completion and it is a flag with an = it will be completed
+    # but we don't want a space after the =
+    if [[ "${#COMPREPLY[@]}" -eq "1" ]] && [[ $(type -t compopt) = "builtin" ]] && [[ "${COMPREPLY[0]}" == --*= ]]; then
+       compopt -o nospace
+    fi
 }
 
 # The arguments should be in the form "ext1|ext2|extn"
-__handle_filename_extension_flag()
+__restic_handle_filename_extension_flag()
 {
     local ext="$1"
     _filedir "@(${ext})"
 }
 
-__handle_subdirs_in_dir_flag()
+__restic_handle_subdirs_in_dir_flag()
 {
     local dir="$1"
     pushd "${dir}" >/dev/null 2>&1 && _filedir -d && popd >/dev/null 2>&1
 }
 
-__handle_flag()
+__restic_handle_flag()
 {
-    __debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
+    __restic_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
 
     # if a command required a flag, and we found it, unset must_have_one_flag()
     local flagname=${words[c]}
@@ -139,30 +145,33 @@ __handle_flag()
     # if the word contained an =
     if [[ ${words[c]} == *"="* ]]; then
         flagvalue=${flagname#*=} # take in as flagvalue after the =
-        flagname=${flagname%%=*} # strip everything after the =
+        flagname=${flagname%=*} # strip everything after the =
         flagname="${flagname}=" # but put the = back
     fi
-    __debug "${FUNCNAME[0]}: looking for ${flagname}"
-    if __contains_word "${flagname}" "${must_have_one_flag[@]}"; then
+    __restic_debug "${FUNCNAME[0]}: looking for ${flagname}"
+    if __restic_contains_word "${flagname}" "${must_have_one_flag[@]}"; then
         must_have_one_flag=()
     fi
 
     # if you set a flag which only applies to this command, don't show subcommands
-    if __contains_word "${flagname}" "${local_nonpersistent_flags[@]}"; then
+    if __restic_contains_word "${flagname}" "${local_nonpersistent_flags[@]}"; then
       commands=()
     fi
 
     # keep flag value with flagname as flaghash
-    if [ -n "${flagvalue}" ] ; then
-        flaghash[${flagname}]=${flagvalue}
-    elif [ -n "${words[ $((c+1)) ]}" ] ; then
-        flaghash[${flagname}]=${words[ $((c+1)) ]}
-    else
-        flaghash[${flagname}]="true" # pad "true" for bool flag
+    # flaghash variable is an associative array which is only supported in bash > 3.
+    if [[ -z "${BASH_VERSION}" || "${BASH_VERSINFO[0]}" -gt 3 ]]; then
+        if [ -n "${flagvalue}" ] ; then
+            flaghash[${flagname}]=${flagvalue}
+        elif [ -n "${words[ $((c+1)) ]}" ] ; then
+            flaghash[${flagname}]=${words[ $((c+1)) ]}
+        else
+            flaghash[${flagname}]="true" # pad "true" for bool flag
+        fi
     fi
 
     # skip the argument to a two word flag
-    if __contains_word "${words[c]}" "${two_word_flags[@]}"; then
+    if __restic_contains_word "${words[c]}" "${two_word_flags[@]}"; then
         c=$((c+1))
         # if we are looking for a flags value, don't show commands
         if [[ $c -eq $cword ]]; then
@@ -174,13 +183,13 @@ __handle_flag()
 
 }
 
-__handle_noun()
+__restic_handle_noun()
 {
-    __debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
+    __restic_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
 
-    if __contains_word "${words[c]}" "${must_have_one_noun[@]}"; then
+    if __restic_contains_word "${words[c]}" "${must_have_one_noun[@]}"; then
         must_have_one_noun=()
-    elif __contains_word "${words[c]}" "${noun_aliases[@]}"; then
+    elif __restic_contains_word "${words[c]}" "${noun_aliases[@]}"; then
         must_have_one_noun=()
     fi
 
@@ -188,47 +197,58 @@ __handle_noun()
     c=$((c+1))
 }
 
-__handle_command()
+__restic_handle_command()
 {
-    __debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
+    __restic_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
 
     local next_command
     if [[ -n ${last_command} ]]; then
         next_command="_${last_command}_${words[c]//:/__}"
     else
         if [[ $c -eq 0 ]]; then
-            next_command="_$(basename "${words[c]//:/__}")"
+            next_command="_restic_root_command"
         else
             next_command="_${words[c]//:/__}"
         fi
     fi
     c=$((c+1))
-    __debug "${FUNCNAME[0]}: looking for ${next_command}"
+    __restic_debug "${FUNCNAME[0]}: looking for ${next_command}"
     declare -F "$next_command" >/dev/null && $next_command
 }
 
-__handle_word()
+__restic_handle_word()
 {
     if [[ $c -ge $cword ]]; then
-        __handle_reply
+        __restic_handle_reply
         return
     fi
-    __debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
+    __restic_debug "${FUNCNAME[0]}: c is $c words[c] is ${words[c]}"
     if [[ "${words[c]}" == -* ]]; then
-        __handle_flag
-    elif __contains_word "${words[c]}" "${commands[@]}"; then
-        __handle_command
-    elif [[ $c -eq 0 ]] && __contains_word "$(basename "${words[c]}")" "${commands[@]}"; then
-        __handle_command
+        __restic_handle_flag
+    elif __restic_contains_word "${words[c]}" "${commands[@]}"; then
+        __restic_handle_command
+    elif [[ $c -eq 0 ]]; then
+        __restic_handle_command
+    elif __restic_contains_word "${words[c]}" "${command_aliases[@]}"; then
+        # aliashash variable is an associative array which is only supported in bash > 3.
+        if [[ -z "${BASH_VERSION}" || "${BASH_VERSINFO[0]}" -gt 3 ]]; then
+            words[c]=${aliashash[${words[c]}]}
+            __restic_handle_command
+        else
+            __restic_handle_noun
+        fi
     else
-        __handle_noun
+        __restic_handle_noun
     fi
-    __handle_word
+    __restic_handle_word
 }
 
 _restic_backup()
 {
     last_command="restic_backup"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -254,8 +274,8 @@ _restic_backup()
     flags+=("--help")
     flags+=("-h")
     local_nonpersistent_flags+=("--help")
-    flags+=("--hostname=")
-    local_nonpersistent_flags+=("--hostname=")
+    flags+=("--host=")
+    local_nonpersistent_flags+=("--host=")
     flags+=("--one-file-system")
     flags+=("-x")
     local_nonpersistent_flags+=("--one-file-system")
@@ -287,6 +307,57 @@ _restic_backup()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
+
+    must_have_one_flag=()
+    must_have_one_noun=()
+    noun_aliases=()
+}
+
+_restic_cache()
+{
+    last_command="restic_cache"
+
+    command_aliases=()
+
+    commands=()
+
+    flags=()
+    two_word_flags=()
+    local_nonpersistent_flags=()
+    flags_with_completion=()
+    flags_completion=()
+
+    flags+=("--cleanup")
+    local_nonpersistent_flags+=("--cleanup")
+    flags+=("--help")
+    flags+=("-h")
+    local_nonpersistent_flags+=("--help")
+    flags+=("--max-age=")
+    local_nonpersistent_flags+=("--max-age=")
+    flags+=("--no-size")
+    local_nonpersistent_flags+=("--no-size")
+    flags+=("--cacert=")
+    flags+=("--cache-dir=")
+    flags+=("--cleanup-cache")
+    flags+=("--json")
+    flags+=("--limit-download=")
+    flags+=("--limit-upload=")
+    flags+=("--no-cache")
+    flags+=("--no-lock")
+    flags+=("--option=")
+    two_word_flags+=("-o")
+    flags+=("--password-file=")
+    two_word_flags+=("-p")
+    flags+=("--quiet")
+    flags+=("-q")
+    flags+=("--repo=")
+    two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -296,6 +367,9 @@ _restic_backup()
 _restic_cat()
 {
     last_command="restic_cat"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -323,6 +397,9 @@ _restic_cat()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -332,6 +409,9 @@ _restic_cat()
 _restic_check()
 {
     last_command="restic_check"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -347,6 +427,8 @@ _restic_check()
     local_nonpersistent_flags+=("--help")
     flags+=("--read-data")
     local_nonpersistent_flags+=("--read-data")
+    flags+=("--read-data-subset=")
+    local_nonpersistent_flags+=("--read-data-subset=")
     flags+=("--with-cache")
     local_nonpersistent_flags+=("--with-cache")
     flags+=("--cacert=")
@@ -365,6 +447,9 @@ _restic_check()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -374,6 +459,9 @@ _restic_check()
 _restic_diff()
 {
     last_command="restic_diff"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -403,6 +491,9 @@ _restic_diff()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -412,6 +503,9 @@ _restic_diff()
 _restic_dump()
 {
     last_command="restic_dump"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -446,6 +540,9 @@ _restic_dump()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -455,6 +552,9 @@ _restic_dump()
 _restic_find()
 {
     last_command="restic_find"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -463,6 +563,8 @@ _restic_find()
     flags_with_completion=()
     flags_completion=()
 
+    flags+=("--blob")
+    local_nonpersistent_flags+=("--blob")
     flags+=("--help")
     flags+=("-h")
     local_nonpersistent_flags+=("--help")
@@ -481,13 +583,19 @@ _restic_find()
     flags+=("--oldest=")
     two_word_flags+=("-O")
     local_nonpersistent_flags+=("--oldest=")
+    flags+=("--pack")
+    local_nonpersistent_flags+=("--pack")
     flags+=("--path=")
     local_nonpersistent_flags+=("--path=")
+    flags+=("--show-pack-id")
+    local_nonpersistent_flags+=("--show-pack-id")
     flags+=("--snapshot=")
     two_word_flags+=("-s")
     local_nonpersistent_flags+=("--snapshot=")
     flags+=("--tag=")
     local_nonpersistent_flags+=("--tag=")
+    flags+=("--tree")
+    local_nonpersistent_flags+=("--tree")
     flags+=("--cacert=")
     flags+=("--cache-dir=")
     flags+=("--cleanup-cache")
@@ -504,6 +612,9 @@ _restic_find()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -513,6 +624,9 @@ _restic_find()
 _restic_forget()
 {
     last_command="restic_forget"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -539,12 +653,12 @@ _restic_forget()
     flags+=("--keep-yearly=")
     two_word_flags+=("-y")
     local_nonpersistent_flags+=("--keep-yearly=")
+    flags+=("--keep-within=")
+    local_nonpersistent_flags+=("--keep-within=")
     flags+=("--keep-tag=")
     local_nonpersistent_flags+=("--keep-tag=")
     flags+=("--host=")
     local_nonpersistent_flags+=("--host=")
-    flags+=("--hostname=")
-    local_nonpersistent_flags+=("--hostname=")
     flags+=("--tag=")
     local_nonpersistent_flags+=("--tag=")
     flags+=("--path=")
@@ -579,6 +693,9 @@ _restic_forget()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -588,6 +705,9 @@ _restic_forget()
 _restic_generate()
 {
     last_command="restic_generate"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -621,6 +741,9 @@ _restic_generate()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -630,6 +753,9 @@ _restic_generate()
 _restic_init()
 {
     last_command="restic_init"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -657,6 +783,9 @@ _restic_init()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -666,6 +795,9 @@ _restic_init()
 _restic_key()
 {
     last_command="restic_key"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -677,6 +809,8 @@ _restic_key()
     flags+=("--help")
     flags+=("-h")
     local_nonpersistent_flags+=("--help")
+    flags+=("--new-password-file=")
+    local_nonpersistent_flags+=("--new-password-file=")
     flags+=("--cacert=")
     flags+=("--cache-dir=")
     flags+=("--cleanup-cache")
@@ -693,6 +827,9 @@ _restic_key()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -702,6 +839,9 @@ _restic_key()
 _restic_list()
 {
     last_command="restic_list"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -729,6 +869,9 @@ _restic_list()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -738,6 +881,9 @@ _restic_list()
 _restic_ls()
 {
     last_command="restic_ls"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -757,6 +903,8 @@ _restic_ls()
     local_nonpersistent_flags+=("--long")
     flags+=("--path=")
     local_nonpersistent_flags+=("--path=")
+    flags+=("--recursive")
+    local_nonpersistent_flags+=("--recursive")
     flags+=("--tag=")
     local_nonpersistent_flags+=("--tag=")
     flags+=("--cacert=")
@@ -775,6 +923,9 @@ _restic_ls()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -784,6 +935,9 @@ _restic_ls()
 _restic_migrate()
 {
     last_command="restic_migrate"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -814,6 +968,9 @@ _restic_migrate()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -823,6 +980,9 @@ _restic_migrate()
 _restic_mount()
 {
     last_command="restic_mount"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -845,6 +1005,8 @@ _restic_mount()
     local_nonpersistent_flags+=("--owner-root")
     flags+=("--path=")
     local_nonpersistent_flags+=("--path=")
+    flags+=("--snapshot-template=")
+    local_nonpersistent_flags+=("--snapshot-template=")
     flags+=("--tag=")
     local_nonpersistent_flags+=("--tag=")
     flags+=("--cacert=")
@@ -863,6 +1025,9 @@ _restic_mount()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -872,6 +1037,9 @@ _restic_mount()
 _restic_prune()
 {
     last_command="restic_prune"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -899,6 +1067,9 @@ _restic_prune()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -908,6 +1079,9 @@ _restic_prune()
 _restic_rebuild-index()
 {
     last_command="restic_rebuild-index"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -935,6 +1109,9 @@ _restic_rebuild-index()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -944,6 +1121,9 @@ _restic_rebuild-index()
 _restic_restore()
 {
     last_command="restic_restore"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -971,6 +1151,8 @@ _restic_restore()
     flags+=("--target=")
     two_word_flags+=("-t")
     local_nonpersistent_flags+=("--target=")
+    flags+=("--verify")
+    local_nonpersistent_flags+=("--verify")
     flags+=("--cacert=")
     flags+=("--cache-dir=")
     flags+=("--cleanup-cache")
@@ -987,6 +1169,53 @@ _restic_restore()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
+
+    must_have_one_flag=()
+    must_have_one_noun=()
+    noun_aliases=()
+}
+
+_restic_self-update()
+{
+    last_command="restic_self-update"
+
+    command_aliases=()
+
+    commands=()
+
+    flags=()
+    two_word_flags=()
+    local_nonpersistent_flags=()
+    flags_with_completion=()
+    flags_completion=()
+
+    flags+=("--help")
+    flags+=("-h")
+    local_nonpersistent_flags+=("--help")
+    flags+=("--output=")
+    local_nonpersistent_flags+=("--output=")
+    flags+=("--cacert=")
+    flags+=("--cache-dir=")
+    flags+=("--cleanup-cache")
+    flags+=("--json")
+    flags+=("--limit-download=")
+    flags+=("--limit-upload=")
+    flags+=("--no-cache")
+    flags+=("--no-lock")
+    flags+=("--option=")
+    two_word_flags+=("-o")
+    flags+=("--password-file=")
+    two_word_flags+=("-p")
+    flags+=("--quiet")
+    flags+=("-q")
+    flags+=("--repo=")
+    two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -996,6 +1225,9 @@ _restic_restore()
 _restic_snapshots()
 {
     last_command="restic_snapshots"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -1035,6 +1267,55 @@ _restic_snapshots()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
+
+    must_have_one_flag=()
+    must_have_one_noun=()
+    noun_aliases=()
+}
+
+_restic_stats()
+{
+    last_command="restic_stats"
+
+    command_aliases=()
+
+    commands=()
+
+    flags=()
+    two_word_flags=()
+    local_nonpersistent_flags=()
+    flags_with_completion=()
+    flags_completion=()
+
+    flags+=("--help")
+    flags+=("-h")
+    local_nonpersistent_flags+=("--help")
+    flags+=("--host=")
+    local_nonpersistent_flags+=("--host=")
+    flags+=("--mode=")
+    local_nonpersistent_flags+=("--mode=")
+    flags+=("--cacert=")
+    flags+=("--cache-dir=")
+    flags+=("--cleanup-cache")
+    flags+=("--json")
+    flags+=("--limit-download=")
+    flags+=("--limit-upload=")
+    flags+=("--no-cache")
+    flags+=("--no-lock")
+    flags+=("--option=")
+    two_word_flags+=("-o")
+    flags+=("--password-file=")
+    two_word_flags+=("-p")
+    flags+=("--quiet")
+    flags+=("-q")
+    flags+=("--repo=")
+    two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -1044,6 +1325,9 @@ _restic_snapshots()
 _restic_tag()
 {
     last_command="restic_tag"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -1084,6 +1368,9 @@ _restic_tag()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -1093,6 +1380,9 @@ _restic_tag()
 _restic_unlock()
 {
     last_command="restic_unlock"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -1122,6 +1412,9 @@ _restic_unlock()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -1131,6 +1424,9 @@ _restic_unlock()
 _restic_version()
 {
     last_command="restic_version"
+
+    command_aliases=()
+
     commands=()
 
     flags=()
@@ -1158,17 +1454,24 @@ _restic_version()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
     noun_aliases=()
 }
 
-_restic()
+_restic_root_command()
 {
     last_command="restic"
+
+    command_aliases=()
+
     commands=()
     commands+=("backup")
+    commands+=("cache")
     commands+=("cat")
     commands+=("check")
     commands+=("diff")
@@ -1185,7 +1488,9 @@ _restic()
     commands+=("prune")
     commands+=("rebuild-index")
     commands+=("restore")
+    commands+=("self-update")
     commands+=("snapshots")
+    commands+=("stats")
     commands+=("tag")
     commands+=("unlock")
     commands+=("version")
@@ -1215,6 +1520,9 @@ _restic()
     flags+=("-q")
     flags+=("--repo=")
     two_word_flags+=("-r")
+    flags+=("--tls-client-cert=")
+    flags+=("--verbose")
+    flags+=("-v")
 
     must_have_one_flag=()
     must_have_one_noun=()
@@ -1225,10 +1533,11 @@ __start_restic()
 {
     local cur prev words cword
     declare -A flaghash 2>/dev/null || :
+    declare -A aliashash 2>/dev/null || :
     if declare -F _init_completion >/dev/null 2>&1; then
         _init_completion -s || return
     else
-        __my_init_completion -n "=" || return
+        __restic_init_completion -n "=" || return
     fi
 
     local c=0
@@ -1243,7 +1552,7 @@ __start_restic()
     local last_command
     local nouns=()
 
-    __handle_word
+    __restic_handle_word
 }
 
 if [[ $(type -t compopt) = "builtin" ]]; then
