@@ -134,7 +134,7 @@ func (node Node) GetExtendedAttribute(a string) []byte {
 	return nil
 }
 
-// CreateAt creates the node at the given path and restores all the meta data.
+// CreateAt creates the node at the given path but does NOT restore node meta data.
 func (node *Node) CreateAt(ctx context.Context, path string, repo Repository, idx *HardlinkIndex) error {
 	debug.Log("create node %v at %v", node.Name, path)
 
@@ -169,6 +169,11 @@ func (node *Node) CreateAt(ctx context.Context, path string, repo Repository, id
 		return errors.Errorf("filetype %q not implemented!\n", node.Type)
 	}
 
+	return nil
+}
+
+// RestoreMetadata restores node metadata
+func (node Node) RestoreMetadata(path string) error {
 	err := node.restoreMetadata(path)
 	if err != nil {
 		debug.Log("restoreMetadata(%s) error %v", path, err)
@@ -181,7 +186,16 @@ func (node Node) restoreMetadata(path string) error {
 	var firsterr error
 
 	if err := lchown(path, int(node.UID), int(node.GID)); err != nil {
-		firsterr = errors.Wrap(err, "Lchown")
+		// Like "cp -a" and "rsync -a" do, we only report lchown permission errors
+		// if we run as root.
+		// On Windows, Geteuid always returns -1, and we always report lchown
+		// permission errors.
+		if os.Geteuid() > 0 && os.IsPermission(err) {
+			debug.Log("not running as root, ignoring lchown permission error for %v: %v",
+				path, err)
+		} else {
+			firsterr = errors.Wrap(err, "Lchown")
+		}
 	}
 
 	if node.Type != "symlink" {
@@ -192,12 +206,10 @@ func (node Node) restoreMetadata(path string) error {
 		}
 	}
 
-	if node.Type != "dir" {
-		if err := node.RestoreTimestamps(path); err != nil {
-			debug.Log("error restoring timestamps for dir %v: %v", path, err)
-			if firsterr != nil {
-				firsterr = err
-			}
+	if err := node.RestoreTimestamps(path); err != nil {
+		debug.Log("error restoring timestamps for dir %v: %v", path, err)
+		if firsterr != nil {
+			firsterr = err
 		}
 	}
 
@@ -285,9 +297,9 @@ func (node Node) createFileAt(ctx context.Context, path string, repo Repository,
 func (node Node) writeNodeContent(ctx context.Context, repo Repository, f *os.File) error {
 	var buf []byte
 	for _, id := range node.Content {
-		size, err := repo.LookupBlobSize(id, DataBlob)
-		if err != nil {
-			return err
+		size, found := repo.LookupBlobSize(id, DataBlob)
+		if !found {
+			return errors.Errorf("id %v not found in repository", id)
 		}
 
 		buf = buf[:cap(buf)]
@@ -517,45 +529,6 @@ func (node Node) sameExtendedAttributes(other Node) bool {
 	return true
 }
 
-// IsNewer returns true of the file has been updated since the last Stat().
-func (node *Node) IsNewer(path string, fi os.FileInfo) bool {
-	if node.Type != "file" {
-		debug.Log("node %v is newer: not file", path)
-		return true
-	}
-
-	tpe := nodeTypeFromFileInfo(fi)
-	if node.Name != fi.Name() || node.Type != tpe {
-		debug.Log("node %v is newer: name or type changed", path)
-		return true
-	}
-
-	size := uint64(fi.Size())
-
-	extendedStat, ok := toStatT(fi.Sys())
-	if !ok {
-		if !node.ModTime.Equal(fi.ModTime()) ||
-			node.Size != size {
-			debug.Log("node %v is newer: timestamp or size changed", path)
-			return true
-		}
-		return false
-	}
-
-	inode := extendedStat.ino()
-
-	if !node.ModTime.Equal(fi.ModTime()) ||
-		!node.ChangeTime.Equal(changeTime(extendedStat)) ||
-		node.Inode != uint64(inode) ||
-		node.Size != size {
-		debug.Log("node %v is newer: timestamp, size or inode changed", path)
-		return true
-	}
-
-	debug.Log("node %v is not newer", path)
-	return false
-}
-
 func (node *Node) fillUser(stat statT) error {
 	node.UID = stat.uid()
 	node.GID = stat.gid()
@@ -635,6 +608,10 @@ func lookupGroup(gid string) (string, error) {
 func (node *Node) fillExtra(path string, fi os.FileInfo) error {
 	stat, ok := toStatT(fi.Sys())
 	if !ok {
+		// fill minimal info with current values for uid, gid
+		node.UID = uint32(os.Getuid())
+		node.GID = uint32(os.Getgid())
+		node.ChangeTime = node.ModTime
 		return nil
 	}
 
